#ifndef UTILITY_H
#define UTILITY_H

#include <algorithm>
#include <iterator>
#include <numeric>
#include <vector>

namespace Utilities {

template <class ActivationFunction>
std::vector<float> feed_layer(std::vector<float>::iterator weight_start,
                              std::vector<float>::iterator weight_end,
                              std::vector<float> &A,
                              ActivationFunction activation_func) {
  // Calculate the new A vector from the current weights
  std::vector<float> Anew;
  Anew.reserve(std::distance(weight_start, weight_end));
  std::transform(
      weight_start, weight_end, std::back_inserter(Anew), [&A](float weight) {
        float summed_weight = std::accumulate(
            A.begin(), A.end(), 0.0f,
            [&weight](float acc, float a) { return acc + a * weight; });
        return summed_weight;
      });
  activation_func(Anew);
  return Anew;
};

} // namespace Utilities
#endif
